/**********************************************************************
 *
 * gsdltimes.cpp -- functions to deal with time
 * Copyright (C) 1999  DigiLib Systems Limited, New Zealand
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: gsdltimes.cpp 371 1999-07-11 10:52:56Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.1  1999/07/11 10:52:56  rjmcnab
   Initial revision.

 */


#include "gsdltimes.h"


// returns "" if an error occurs
text_t time2text (time_t time) {
  tm *tm_ptr = localtime(&time);
  if (tm_ptr == NULL) return "";

  // year
  text_t timestr(tm_ptr->tm_year+1900);
  timestr.push_back('/');

  // month (note the +1)
  if ((tm_ptr->tm_mon+1) < 10) timestr.push_back('0');
  timestr += (tm_ptr->tm_mon+1);
  timestr.push_back('/');

  // day
  if (tm_ptr->tm_mday < 10) timestr.push_back('0');
  timestr += tm_ptr->tm_mday;
  timestr.push_back(' ');

  // hour
  if (tm_ptr->tm_hour < 10) timestr.push_back('0');
  timestr += tm_ptr->tm_hour;
  timestr.push_back(':');

  // minutes
  if (tm_ptr->tm_min < 10) timestr.push_back('0');
  timestr += tm_ptr->tm_min;
  timestr.push_back(':');

  // seconds
  if (tm_ptr->tm_sec < 10) timestr.push_back('0');
  timestr += tm_ptr->tm_sec;

  return timestr;
}

// returns -1 if an error occurs
time_t text2time (const text_t &timestr) {
  tm timetm;
  text_t tempstr;
  
  text_t::const_iterator time_here = timestr.begin();
  text_t::const_iterator time_end = timestr.end();

  // year
  if (time_here == time_end) return (time_t) -1;
  time_here = getdelimitstr (time_here, time_end, '/', tempstr);
  timetm.tm_year = tempstr.getint() - 1900;
  if (timetm.tm_year < 0) return (time_t) -1;

  // month (note the -1)
  if (time_here == time_end) return (time_t) -1;
  time_here = getdelimitstr (time_here, time_end, '/', tempstr);
  timetm.tm_mon = tempstr.getint() - 1;
  if (timetm.tm_mon < 0 || timetm.tm_mon > 11)
    return (time_t) -1;
  
  // day
  if (time_here == time_end) return (time_t) -1;
  time_here = getdelimitstr (time_here, time_end, ' ', tempstr);
  timetm.tm_mday = tempstr.getint();
  if (timetm.tm_mday < 1 || timetm.tm_mday > 31)
    return (time_t) -1;
  
  // hour
  if (time_here == time_end) return (time_t) -1;
  time_here = getdelimitstr (time_here, time_end, ':', tempstr);
  timetm.tm_hour = tempstr.getint();
  if (timetm.tm_hour < 0 || timetm.tm_hour > 23)
    return (time_t) -1;
  
  // min
  if (time_here == time_end) return (time_t) -1;
  time_here = getdelimitstr (time_here, time_end, ':', tempstr);
  timetm.tm_min = tempstr.getint();
  if (timetm.tm_min < 0 || timetm.tm_min > 59)
    return (time_t) -1;
  
  // sec
  if (time_here == time_end) return (time_t) -1;
  time_here = getdelimitstr (time_here, time_end, ':', tempstr);
  timetm.tm_sec = tempstr.getint();
  if (timetm.tm_sec < 0 || timetm.tm_sec > 59)
    return (time_t) -1;
  
  return mktime (&timetm);
}
