# functions to handle classifiers

package classify;

require util;


$next_classify_num = 1;

# load_classifier will load one classifier. $classinfo is an
# array containing information about the classifier to load. The
# first element of the array is the name of the classifier, the
# rest of the elements are classifier specific.
sub load_classifier {
    my ($classinfo) = @_;

    # get the classifier name
    my $classname = shift(@$classinfo);
    return undef unless defined $classname;
    
    # find the classifier
    my $colclassname = &util::filename_cat($ENV{'GSDLCOLLECTDIR'},"perllib/classify", 
					   "${classname}.pm");
    my $mainclassname = &util::filename_cat($ENV{'GSDLHOME'},"perllib/classify", 
					    "${classname}.pm");
    if (-e $colclassname) { require $colclassname; }
    elsif (-e $mainclassname) { require $mainclassname; }
    else { die "ERROR - couldn't find classifier \"$classname\"\n"; }

    # create the classify object
    my ($classobj);
    map { $_ = "\"$_\""; } @$classinfo;
    my $options = join (",", @$classinfo);
    eval ("\$classobj = new $classname($options)");
    die "$@" if $@;

    return $classobj;
}

# init_classifiers resets all the classifiers and readys them to process
# the documents.
sub init_classifiers {
    my ($classifiers) = @_;
    
    foreach $classobj (@$classifiers) {
	$classobj->init();
    }
}

# classify_doc lets each of the classifiers classify a document
sub classify_doc {
    my ($classifiers, $doc_obj) = @_;
    
    foreach $classobj (@$classifiers) {
	$classobj->classify($doc_obj);
    }
}

# output_classify_info outputs all the info needed for the classification
# to the gdbm
sub output_classify_info {
    my ($classifiers, $handle, $allclassifications) = @_;
#    $handle = "main::STDOUT";

    # create a classification containing all the info
    my $classifyinfo = {'classifyOID'=>'browse',
			'contains'=>[]};

    # get each of the classifications
    foreach $classobj (@$classifiers) {
	my $tempinfo = $classobj->get_classify_info();
	$tempinfo->{'classifyOID'} = "CL$next_classify_num";
	$next_classify_num++;
	push (@{$classifyinfo->{'contains'}}, $tempinfo);
    }

    &print_classify_info ($handle, $classifyinfo, "", $allclassifications);
}

sub print_classify_info {
    my ($handle, $classifyinfo, $OID, $allclassifications) = @_;
    $OID =~ s/^\.+//; # just for good luck

    # book information is printed elsewhere
    return if (defined ($classifyinfo->{'OID'}));
 
    # don't want empty classifications
    if ($allclassifications || &clean_contents ($classifyinfo)) {
   
	$OID = $classifyinfo->{'classifyOID'} if defined ($classifyinfo->{'classifyOID'});
	
	my $outputtext = "[$OID]\n";
	$outputtext .= "<doctype>classify\n";
	$outputtext .= "<hastxt>0\n";
	$outputtext .= "<classifytype>$classifyinfo->{'classifytype'}\n" if defined $classifyinfo->{'classifytype'};
	$outputtext .= "<Title>$classifyinfo->{'Title'}\n" if defined $classifyinfo->{'Title'};
	
	$outputtext .= "<contains>";
	
	my $next_subOID = 1;
	my $first = 1;
	foreach $tempinfo (@{$classifyinfo->{'contains'}}) {
	    # empty contents were made undefined by clean_contents()
	    next unless defined $tempinfo;
	    
	    $outputtext .= ";" unless $first;
	    $first = 0;
	    
	    if (defined ($tempinfo->{'classifyOID'})) {
		$outputtext .= $tempinfo->{'classifyOID'};
		&print_classify_info ($handle, $tempinfo, $tempinfo->{'classifyOID'},
				      $allclassifications);
	    } elsif (defined ($tempinfo->{'OID'})) {
		$outputtext .= $tempinfo->{'OID'};
		# note: we don't want to print the contents of the books
	    } else {
		$outputtext .= "\".$next_subOID";
		&print_classify_info ($handle, $tempinfo, "$OID.$next_subOID",
				      $allclassifications);
		$next_subOID++;
	    }
	}
	$outputtext .= "\n";
	$outputtext .= '-' x 70 . "\n";
	
	print $handle $outputtext;
    }
}

sub clean_contents {
    my ($classifyinfo) = @_;
    my $has_content = 0;

    foreach $content (@{$classifyinfo->{'contains'}}) {
	if (defined $content->{'OID'}) {
	    # found a book
	    $has_content = 1;
	} elsif (&clean_contents ($content)) {
	    # there's a book somewhere below
	    $has_content = 1;
	} else {
	    # section contains no books so we want to remove 
	    # it from its parents contents
	    $content = undef;
	}
    }
    return $has_content;
}

1;
