# simple list classifier plugin 
# collect.cfg usage: 'List metaelement' or simply 'List'
# the first will create a single list of all documents
# with metaelement metadata (sorted alphabetically by whichever
# metadata element was chosen).
# The second will create a single list of all documents
# which won't be sorted at all - they may well be in 
# some completely random order ;-|


package List;

use sorttools;

sub new {
    my ($class, @options) = @_;

    my $list = [];
    if (defined $options[0]) {
	$list = {};
    }

    return bless {
	'list'=>$list,
	'metaname' => $options[0]
	}, $class;
}

sub init {
    my $self = shift (@_);

    if (defined $self->{'metaname'}) {
	$self->{'list'} = {};
    } else {
	$self->{'list'} = [];
    }
}

sub classify {
    my $self = shift (@_);
    my ($doc_obj) = @_;

    my $doc_OID = $doc_obj->get_OID();

    if (defined $self->{'metaname'}) {
	my $metavalue = $doc_obj->get_metadata_element ($doc_obj->get_top_section(),
						     $self->{'metaname'});
	if (defined $metavalue) {
	    if ($self->{'metaname'} eq 'Creator') {
		&sorttools::format_string_name_english (\$metavalue);
	    } else {
		&sorttools::format_string_english (\$metavalue);
	    }
	    if (defined $self->{'list'}->{$doc_OID}) {
		print STDERR "WARNING: List::classify called multiple times for $doc_OID\n";
	    } 
	    $self->{'list'}->{$doc_OID} = $metavalue;
	}
    } else {
	push (@{$self->{'list'}}, $doc_OID);
    }
}

sub get_classify_info {
    my $self = shift (@_);

    my $classifytitle = "List";
    my $contains = "";
    my @list = ();
    if (defined $self->{'metaname'}) {
	$classifytitle = $self->{'metaname'};
	if (keys %{$self->{'list'}}) {
	    @list = sort {$self->{'list'}->{$a} 
			  cmp $self->{'list'}->{$b};} keys %{$self->{'list'}};
	}
    } else {
	@list = @{$self->{'list'}};
    }

    # organise into classification structure
    my %classifyinfo = ('classifytype'=>'List',
			'Title'=>$classifytitle,
			'contains'=>[]);
    foreach $OID (@list) {
	push (@{$classifyinfo{'contains'}}, {'OID'=>$OID});
    }

    return \%classifyinfo;
}


1;
