
/**********************************************************************
 *
 * collectserver.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: collectserver.cpp 352 1999-07-08 03:58:44Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.10  1999/07/08 03:58:44  sjboddie
   format stuff

   Revision 1.9  1999/06/16 02:00:34  sjboddie
   Few changes to get getParent filter option to return metadata of
   parents as well as current OID

   Revision 1.8  1999/05/10 03:43:47  sjboddie
   lots of changes to lots of files - getting document action going

   Revision 1.7  1999/04/30 02:00:45  sjboddie
   lots of stuff to do with getting documentaction working

   Revision 1.6  1999/04/06 22:20:29  rjmcnab
   Got browsefilter working.

   Revision 1.5  1999/03/31 23:44:44  rjmcnab
   Altered the protocol so that the metadata is part of the filter.

   Revision 1.4  1999/03/09 20:58:50  rjmcnab
   Added dummy filter and metadata results.

   Revision 1.3  1999/03/08 05:07:42  rjmcnab
   Made some alterations to fit with the changes to the comtypes. Added the
   "filteroptdefault" configuration option to alter default filter options.

   Revision 1.2  1999/03/03 23:28:29  sjboddie

   Provided stub functions for the protocol

   Revision 1.1  1999/02/21 22:32:56  rjmcnab

   Initial revision.

 */


#include "collectserver.h"
#include "infodbclass.h"
#include "OIDtools.h"
#include <assert.h>


collectserver::collectserver () {
  configinfo.collection = "null";
}

collectserver::~collectserver () {
}

// configure should be called for each line in the
// configuration files to configure the collection server and everything
// it contains. The configuration should take place just before initialisation.
void collectserver::configure (const text_t &key, const text_tarray &cfgline) {
  if (cfgline.size() >= 1) {
    const text_t &value = cfgline[0];
    if (key == "gsdlhome") configinfo.gsdlhome = value;
    else if (key == "collection") {
      configinfo.collection = value;
      collectinfo.shortInfo.name = value;
    } else if (key == "collectdir") configinfo.collectdir = value;
    else if (key == "host") collectinfo.shortInfo.host = value;
    else if (key == "port") collectinfo.shortInfo.port = value.getint();
    else if (key == "public") {
      if (value == "true") collectinfo.isPublic = true;
      else collectinfo.isPublic = false;
    } else if (key == "beta") {
      if (value == "true") collectinfo.isBeta = true;
      else collectinfo.isBeta = false;
    } else if (key == "builddate") collectinfo.buildDate = value.getint();
    else if (key == "languages") collectinfo.languages = cfgline;
    else if (key == "numdocs") collectinfo.numDocs = value.getint();
    else if (key == "numwords") collectinfo.numWords = value.getint();
    else if (key == "numbytes") collectinfo.numBytes = value.getint();
    else if (key == "format" && cfgline.size() == 2)
      collectinfo.format[cfgline[0]] = cfgline[1];
  }

  // configure the filters
  filtermapclass::iterator filter_here = filters.begin();
  filtermapclass::iterator filter_end = filters.end();
  while (filter_here != filter_end) {
    assert ((*filter_here).second.f != NULL);
    if ((*filter_here).second.f != NULL) 
      (*filter_here).second.f->configure(key, cfgline);

    filter_here++;
  }

  // configure the sources
  sourcelistclass::iterator source_here = sources.begin();
  sourcelistclass::iterator source_end = sources.end();
  while (source_here != source_end) {
    assert ((*source_here).s != NULL);
    if ((*source_here).s != NULL)
      (*source_here).s->configure(key, cfgline);
    
    source_here++;
  }
}

void collectserver::configure (const text_t &key, const text_t &value) {
  text_tarray cfgline;
  cfgline.push_back (value);
  configure(key, cfgline);
}


bool collectserver::init (ostream &logout) {
  // init the filters
  filtermapclass::iterator filter_here = filters.begin();
  filtermapclass::iterator filter_end = filters.end();
  while (filter_here != filter_end) {
    assert ((*filter_here).second.f != NULL);
    if (((*filter_here).second.f != NULL) &&
	!(*filter_here).second.f->init(logout)) return false;
    
    filter_here++;
  }

  // init the sources
  sourcelistclass::iterator source_here = sources.begin();
  sourcelistclass::iterator source_end = sources.end();
  while (source_here != source_end) {
    assert ((*source_here).s != NULL);
    if (((*source_here).s != NULL) &&
	!(*source_here).s->init(logout)) return false;
    
    source_here++;
  }

  return true;
}


void collectserver::get_collectinfo (ColInfoResponse_t &reponse,
		      comerror_t &err, ostream &/*logout*/) {
  reponse = collectinfo; 
  err = noError;
}

void collectserver::get_filterinfo (InfoFiltersResponse_t &response,
				    comerror_t &err, ostream &/*logout*/) {
  response.clear ();

  // get a list of filter names
  filtermapclass::iterator filter_here = filters.begin();
  filtermapclass::iterator filter_end = filters.end();
  while (filter_here != filter_end) {
    response.filterNames.insert ((*filter_here).first);
    filter_here++;
  }

  err = noError;
}

void collectserver::get_filteroptions (const InfoFilterOptionsRequest_t &request,
				       InfoFilterOptionsResponse_t &response, 
				       comerror_t &err, ostream &logout) {
  outconvertclass text_t2ascii;

  filterclass *thisfilter = filters.getfilter(request.filterName);
  if (thisfilter != NULL) {
    thisfilter->get_filteroptions (response, err, logout);
  } else {
    response.clear ();
    err = protocolError;
    logout << text_t2ascii << "Protocol Error: filter options requested for non-existent\n"
	   << "filter \"" << request.filterName << "\".\n\n";
  }
}

void collectserver::filter (FilterRequest_t &request,
			    FilterResponse_t &response,
			    comerror_t &err, ostream &logout) {
  outconvertclass text_t2ascii;

  // translate any ".fc", ".pr" etc. stuff in the docSet
  text_t translatedOID;
  text_tset translatedOIDs;
  text_tset::iterator doc_here = request.docSet.begin();
  text_tset::iterator doc_end = request.docSet.end();
  while (doc_here != doc_end) {
    if (needs_translating (*doc_here)) {
      sourcelistclass::iterator source_here = sources.begin();
      sourcelistclass::iterator source_end = sources.end();
      while (source_here != source_end) {
	assert ((*source_here).s != NULL);
	if (((*source_here).s != NULL) &&
	    ((*source_here).s->translate_OID (*doc_here, translatedOID, err, logout))) {
	  if (err != noError) return;
	  break;
	}
	source_here++;
      }
      translatedOIDs.insert (translatedOID);
    } else {
      translatedOIDs.insert (*doc_here);
    }
    doc_here ++;
  }
  request.docSet = translatedOIDs;

  response.clear();

  filterclass *thisfilter = filters.getfilter(request.filterName);
  if (thisfilter != NULL) {
    // filter the data
    thisfilter->filter (request, response, err, logout);

    // fill in the metadata for each of the OIDs (if it is requested)
    if (request.filterResultOptions & FRmetadata) {
      ResultDocInfo_tarray::iterator resultdoc_here = response.docInfo.begin();
      ResultDocInfo_tarray::iterator resultdoc_end = response.docInfo.end();
      while (resultdoc_here != resultdoc_end) {
	// try each of the sources in turn
	sourcelistclass::iterator source_here = sources.begin();
	sourcelistclass::iterator source_end = sources.end();
	while (source_here != source_end) {
	  assert ((*source_here).s != NULL);
	  if (((*source_here).s != NULL) &&
	      ((*source_here).s->get_metadata(request.requestParams, request.refParams,
					      request.getParents, request.fields, 
					      (*resultdoc_here).OID, (*resultdoc_here).metadata, 
					      err, logout))) {
	    if (err != noError) return;
	    break;
	  }
	  
	  source_here++;
	}
	
	resultdoc_here++;
      }
    }
    
  } else {
    response.clear ();
    err = protocolError;
    logout << text_t2ascii << "Protocol Error: filter options requested for non-existent\n"
	   << "filter \"" << request.filterName << "\".\n\n";
  }
  
  err = noError;
}

void collectserver::get_document (const DocumentRequest_t &request,
				  DocumentResponse_t &response,
				  comerror_t &err, ostream &logout) {

  sourcelistclass::iterator source_here = sources.begin();
  sourcelistclass::iterator source_end = sources.end();
  while (source_here != source_end) {
    assert ((*source_here).s != NULL);
    if (((*source_here).s != NULL) &&
	((*source_here).s->get_document (request.OID, response.doc, err, logout))) {
      if (err != noError) return;
      break;
    }
    source_here++;
  }
}

// thecollectserver remains the property of the calling code but
// should not be deleted until it is removed from this list.
void collectservermapclass::addcollectserver (collectserver *thecollectserver) {
  // can't add a null collection server
  assert (thecollectserver != NULL);
  if (thecollectserver == NULL) return;
  
  // can't add an collection server with no collection name
  assert (!(thecollectserver->get_collection_name()).empty());
  if ((thecollectserver->get_collection_name()).empty()) return;

  collectserverptr cptr;
  cptr.c = thecollectserver;
  collectserverptrs[thecollectserver->get_collection_name()] = cptr;
}

// getcollectserver will return NULL if the collectserver could not be found
collectserver *collectservermapclass::getcollectserver (const text_t &collection) {
  // can't find a collection with no name
  assert (!collection.empty());
  if (collection.empty()) return NULL;

  iterator here = collectserverptrs.find (collection);
  if (here == collectserverptrs.end()) return NULL;
  
  return (*here).second.c;
}

